var XnhdAPI;
(function (XnhdAPI) {
    /**
     * 提供初级音效、音乐的播放、停止、设置声音大小等方法，
     * 音效暂定只播放1次，播放完毕不释放内存，音乐循环播放，但同时只会播放1个音乐；
     */
    class Sound {
        constructor() {
            // private _soundPathArr:string[] = [];
            this._soundInstArr = [];
            this._musicInst = null;
            this.volumeMusic = 35;
            this.volumeSound = 100;
        }
        /**
         * 声音初始化
         * @param soundPathArr 音效文件地址数组（字符串），不要带文件后缀名，apk环境ogg文件，其他环境mp3文件
         * @param musicPath 背景音乐文件地址（字符串），不要带文件后缀名，mp3文件，可为null
         * @param soundVolume 音效音量，默认100，最大100，最小为0
         * @param musicVolume 背景音乐音量，默认35，最大100，最小为0
         */
        init(soundPathArr, musicPath, soundVolume = 100, musicVolume = 35) {
            Laya.SoundManager.autoReleaseSound = false;
            Laya.SoundManager.autoStopMusic = false;
            if (typeof soundVolume === "number") {
                this.volumeSound = soundVolume;
            }
            if (typeof musicVolume === "number") {
                this.volumeMusic = musicVolume;
            }
            this.setMusicVolume(this.volumeMusic);
            this.setSoundVolume(this.volumeSound);
            if (XnhdAPI.Param.platType === 2 && XnhdAPI.Param.channelType === 3) {
                this._musicInst = Laya.Browser.window.qg.createInnerAudioContext();
                this._musicInst.src = musicPath + ".mp3";
                this._musicInst.loop = true;
                this._musicInst.volume = this.volumeMusic / 100;
            }
            else if (XnhdAPI.Param.platType === 2 && XnhdAPI.Param.channelType === 5) {
                this._musicInst = Laya.SoundManager.playSound(musicPath + ".mp3", 0);
            }
        }
        /**
         * 播放音效
         * @param whitchSound 音效地址，不要带文件后缀名
         */
        playSound(whitchSound) {
            if (XnhdAPI.Param.platType === 2 && XnhdAPI.Param.channelType === 3) {
                try {
                    let soundChannel;
                    soundChannel = Laya.Browser.window.qg.createInnerAudioContext();
                    soundChannel.src = whitchSound + ".mp3";
                    soundChannel.loop = false;
                    soundChannel.volume = this.volumeSound / 100;
                    soundChannel.play();
                }
                catch (f) {
                    XnhdAPI.logError("vivo playSound failed >> " + whitchSound);
                }
                return;
            }
            if (XnhdAPI.Param.platType === 2 && XnhdAPI.Param.channelType === 1) {
                let soundChannel;
                for (let i = 0; i < this._soundInstArr.length; i++) {
                    if ([0, 2].indexOf(this._soundInstArr[i].getStatus()) > -1) {
                        soundChannel = this._soundInstArr[i];
                        break;
                    }
                }
                if (soundChannel) { }
                else {
                    soundChannel = new XnhdSoundChannel();
                    this._soundInstArr.push(soundChannel);
                }
                soundChannel.setPropertyAndPlay(whitchSound + ".mp3", this.volumeSound / 100);
                return;
            }
            if (XnhdAPI.Param.platType === 0) {
                // Laya.SoundManager.playSound(whitchSound + ".ogg", 1);
                Laya.SoundManager.playSound(whitchSound + ".mp3", 1);
                return;
            }
            Laya.SoundManager.playSound(whitchSound + ".mp3", 1);
        }
        /**
         * 播放仅有1例的音乐
         * @param whitchMusic 音乐地址，不要带文件后缀名
         */
        playMusic(whitchMusic) {
            if (XnhdAPI.Param.platType === 2 && XnhdAPI.Param.channelType === 3) {
                this._musicInst.play();
                return;
            }
            if (XnhdAPI.Param.platType === 2 && XnhdAPI.Param.channelType === 5) {
                if (this._musicInst) {
                    this._musicInst.resume();
                }
                else {
                    this._musicInst = Laya.SoundManager.playMusic(whitchMusic + ".mp3", 0);
                    // Laya.timer.frameLoop(Math.floor(36.65 * 60), this, () => {
                    //     XnhdAPI.logError("播放停止了")
                    //     this._musicInst.play();
                    // })
                    // Laya.timer.frameLoop(60, this, () => {
                    //     XnhdAPI.logError("播放了:"+this._musicInst.position.toString());
                    // })
                }
                return;
            }
            this._musicInst = Laya.SoundManager.playMusic(whitchMusic + ".mp3", 0);
        }
        setSoundVolume(value) {
            this.volumeSound = value;
            if (XnhdAPI.Param.platType === 2 && XnhdAPI.Param.channelType === 3) {
                return;
            }
            Laya.SoundManager.setSoundVolume(value / 100);
        }
        setMusicVolume(value) {
            this.volumeMusic = value;
            if (XnhdAPI.Param.platType === 2 && XnhdAPI.Param.channelType === 3) {
                if (this._musicInst) {
                    this._musicInst.volume = value / 100;
                }
                return;
            }
            Laya.SoundManager.setMusicVolume(value / 100);
        }
        stopMusic() {
            if (XnhdAPI.Param.platType === 2 && XnhdAPI.Param.channelType === 3) {
                if (this._musicInst) {
                    this._musicInst.stop();
                }
                return;
            }
            if (XnhdAPI.Param.platType === 2 && XnhdAPI.Param.channelType === 5) {
                if (this._musicInst) {
                    this._musicInst.pause();
                }
                return;
            }
            Laya.SoundManager.stopMusic();
        }
        stopAllSound() {
            Laya.SoundManager.stopAllSound();
        }
        stopAll() {
            this.stopAllSound();
            this.stopMusic();
        }
    }
    Sound.Manager = new Sound();
    XnhdAPI.Sound = Sound;
    class XnhdSoundChannel {
        /**
         * 防止某些SB渠道的声音状态不正确导致影响播放（微信、百度）
         * @param index 声音的编号
         */
        constructor() {
            switch (XnhdAPI.Param.channelType) {
                case 1: {
                    this._soundInstance = Laya.Browser.window.wx.createInnerAudioContext();
                    this._soundInstance.onEnded(() => {
                        this._soundStatus = 0;
                    });
                    break;
                }
            }
            this._soundInstance.autoplay = false;
            this._soundInstance.loop = false;
            this._soundInstance.onError(res => {
                this._soundStatus = 2;
                XnhdAPI.logError("播放音效错误，src = " + this._src + " : " + JSON.stringify(res));
            });
        }
        setPropertyAndPlay(src, volume) {
            this._soundInstance.volume = volume;
            this._soundInstance.src = src;
            this._src = src;
            this._soundStatus = 1;
            this._soundInstance.play();
        }
        getStatus() {
            return this._soundStatus;
        }
    }
})(XnhdAPI || (XnhdAPI = {}));
