var XnhdAPI;
(function (XnhdAPI) {
    var WebSocketAPI;
    (function (WebSocketAPI) {
        /**
         * 建立websocket连接
         * @param URL 连接地址
         */
        function websocketConnect(URL) {
            if (Manager.Default) { }
            else {
                Manager.Default = new Manager;
            }
            Manager.Default && Manager.Default.connectURL(URL);
        }
        WebSocketAPI.websocketConnect = websocketConnect;
        /**
         * 关闭websocket连接
         */
        function websocketClose() {
            Manager.Default && Manager.Default.closeConnection();
        }
        WebSocketAPI.websocketClose = websocketClose;
        /**
         * 获取从服务器端接收到的信息，每调用1次函数仅返回1条信息，获得的信息顺序跟服务器返回顺序一致，如当前没有信息，则返回undefined
         */
        function websocketReceiveMsgFunc(receivefunc, openFunc, closeFunc) {
            Manager.Default && Manager.Default.websocketMonitorOn(receivefunc, openFunc, closeFunc);
        }
        WebSocketAPI.websocketReceiveMsgFunc = websocketReceiveMsgFunc;
        function websocketSendMsg(command) {
            Manager.Default && Manager.Default.sendString(command);
        }
        WebSocketAPI.websocketSendMsg = websocketSendMsg;
        /**
         * 检查websocket连接状态，如正常则返回true，否则返回false并输出报错日志
         */
        function websocketCheckConnectionStatus() {
            if (Manager.Default) {
                switch (Manager.Default.checkConnectStatus()) {
                    case 0: {
                        XnhdAPI.logError("webSocket：检查连接状态-正在建立连接");
                        return false;
                    }
                    case 1: {
                        return true;
                    }
                    case 2: {
                        XnhdAPI.logError("webSocket：检查连接状态-当前连接已关闭");
                        return false;
                    }
                    default: {
                        XnhdAPI.logError("webSocket：检查连接状态-未建立连接");
                        return false;
                    }
                }
            }
            else {
                return false;
            }
        }
        WebSocketAPI.websocketCheckConnectionStatus = websocketCheckConnectionStatus;
        /**
         * websocket管理类
         */
        class Manager {
            constructor() {
            }
            connectURL(URL) {
                this.socketStatus = 0;
                if (XnhdAPI.Param.platType === 2 && XnhdAPI.Param.channelType === 1) {
                    if (this.socket) { }
                    else {
                        this.socket = XnhdAPI.WxLiteGameAPI.websocketConnect(URL);
                    }
                }
                else {
                    if (this.socket) { }
                    else {
                        this.socket = new Laya.Socket();
                        this.socket.endian = Laya.Byte.LITTLE_ENDIAN;
                    }
                    this.socket.connectByUrl(URL);
                }
            }
            websocketMonitorOn(receiveFunc, openSocket, closeFunc) {
                this.receiveFunc = receiveFunc;
                this.closeFunc = closeFunc;
                this.openSocket = openSocket;
                if (XnhdAPI.Param.platType === 2 && XnhdAPI.Param.channelType === 1) {
                    XnhdAPI.WxLiteGameAPI.websocketMonitorOn(e => { this.receiveHandler(e); }, () => { this.openHandler(); }, () => {
                        this.closeHandler();
                    }, () => { this.errorHandler(); });
                }
                else {
                    this.socket.on(Laya.Event.OPEN, this, this.openHandler);
                    this.socket.on(Laya.Event.MESSAGE, this, this.receiveHandler);
                    this.socket.on(Laya.Event.CLOSE, this, this.closeHandler);
                    this.socket.on(Laya.Event.ERROR, this, this.errorHandler);
                }
            }
            openHandler() {
                //正确建立连接；
                this.socketStatus = 1;
                this.openSocket && this.openSocket();
                XnhdAPI.logDebug("webSocket:正确建立连接");
            }
            receiveHandler(msg = null) {
                ///接收到数据触发函数
                this.receiveFunc && this.receiveFunc(msg);
                XnhdAPI.logDebug("webSocket:接收到数据: " + (msg ? msg : ""));
            }
            closeHandler() {
                //关闭事件
                this.socketStatus = 2;
                this.closeFunc && this.closeFunc();
                XnhdAPI.logDebug("webSocket:关闭连接");
            }
            errorHandler() {
                //连接出错
                XnhdAPI.logError("webSocket:连接出错");
            }
            sendString(commad) {
                if (this.socketStatus === 1) {
                    if (XnhdAPI.Param.platType === 2 && XnhdAPI.Param.channelType === 1) {
                        XnhdAPI.WxLiteGameAPI.websocketSendMessage(commad);
                    }
                    else {
                        this.socket.send(commad);
                    }
                }
                else if (this.socketStatus === 2) {
                    XnhdAPI.logError("webSocket：send请求出错，原因：连接已关闭");
                }
                else if (this.socketStatus === 0) {
                    XnhdAPI.logError("webSocket：send请求出错，原因：正在连接中");
                }
                else {
                    XnhdAPI.logError("webSocket：send请求出错，原因：未建立连接");
                }
            }
            closeConnection() {
                if (XnhdAPI.Param.platType === 2 && XnhdAPI.Param.channelType === 1) {
                    XnhdAPI.WxLiteGameAPI.websocketclose();
                    this.socket = null;
                }
                else {
                    this.socket.close();
                }
            }
            checkConnectStatus() {
                return this.socketStatus;
            }
        }
    })(WebSocketAPI = XnhdAPI.WebSocketAPI || (XnhdAPI.WebSocketAPI = {}));
})(XnhdAPI || (XnhdAPI = {}));
