/**
 * 控制声音
 */
var core;
(function (core) {
    class AudioAndSetting {
        constructor() {
            this._soundPathArr = [
                "unpack/sound/click",
                "unpack/sound/put",
                "unpack/sound/putdown",
                "unpack/sound/levelUp",
                "unpack/sound/gameOver",
                "unpack/sound/bomb",
                "unpack/sound/hugeReward1",
                "unpack/sound/hugeReward2",
            ];
            this._music = "unpack/sound/bg";
            this.volumeMusic = 70;
            this.volumeSound = 100;
            this.vibrate = true;
        }
        sound_putDown() {
            this.playSound(this._soundPathArr[2]);
        }
        soundClick() {
            this.playSound(this._soundPathArr[0]);
        }
        sound_pickUp() {
            this.playSound(this._soundPathArr[1]);
        }
        sound_levelUp() {
            this.playSound(this._soundPathArr[3]);
        }
        sound_gameOver() {
            this.playSound(this._soundPathArr[4]);
        }
        sound_bomb() {
            this.playSound(this._soundPathArr[5]);
        }
        sound_startTurntable() {
            this.playSound(this._soundPathArr[6]);
        }
        sound_stopTurntable() {
            this.playSound(this._soundPathArr[7]);
        }
        soundCombo(num) {
            num > 8 && (num = 8);
            this.playSound("unpack/sound/eliminate_" + num.toString());
        }
        soundCongratulate(num) {
            num > 5 && (num = 5);
            this.playSound("unpack/sound/congratulate_" + num.toString());
        }
        playMusic() {
            XnhdAPI.Sound.Manager.playMusic(this._music);
        }
        playSound(whitchSound) {
            XnhdAPI.Sound.Manager.playSound(whitchSound);
        }
        init() {
            Laya.SoundManager.autoReleaseSound = false;
            let res = XnhdAPI.readDataFromCache("setting");
            if (res && res.length) {
                try {
                    let jsonData = JSON.parse(res);
                    if (jsonData) {
                        if (jsonData.music > 1 || jsonData.music === 0) {
                            this.volumeMusic = jsonData.music;
                        }
                        if (jsonData.sound > 1 || jsonData.sound === 0) {
                            this.volumeSound = jsonData.sound;
                        }
                        if (typeof jsonData.vibrate === 'boolean') {
                            this.vibrate = jsonData.vibrate;
                        }
                        if (typeof jsonData.lang === 'number' && jsonData.lang > -1) {
                            core.GameDef.languageIndex = jsonData.lang;
                        }
                    }
                }
                catch (e) {
                    XnhdAPI.logError("error: position: readLocalConfig >>" + e.toString());
                }
            }
            XnhdAPI.Sound.Manager.init(this._soundPathArr, this._music, this.volumeSound, this.volumeMusic);
        }
        writeSettingConfig() {
            XnhdAPI.saveDataToCache("setting", JSON.stringify({
                music: this.volumeMusic,
                sound: this.volumeSound,
                vibrate: this.vibrate,
                lang: core.GameDef.languageIndex,
            }), false, () => {
                XnhdAPI.logDebug("save Audio");
            });
        }
        setSoundVolume(value) {
            this.volumeSound = value;
            XnhdAPI.Sound.Manager.setSoundVolume(value);
            this.writeSettingConfig();
        }
        setMusicVolume(value) {
            this.volumeMusic = value;
            XnhdAPI.Sound.Manager.setMusicVolume(value);
            this.writeSettingConfig();
        }
        setVibrate(isVibrate) {
            this.vibrate = isVibrate;
            this.writeSettingConfig();
        }
        stopMusic() {
            XnhdAPI.Sound.Manager.stopMusic();
        }
        stopSound() {
            XnhdAPI.Sound.Manager.stopAllSound();
        }
        stopAll() {
            XnhdAPI.Sound.Manager.stopAll();
        }
    }
    AudioAndSetting.Manager = new AudioAndSetting();
    core.AudioAndSetting = AudioAndSetting;
})(core || (core = {}));
