var core;
(function (core) {
    let selectCardType;
    (function (selectCardType) {
        selectCardType[selectCardType["areaPos1"] = 1] = "areaPos1";
        selectCardType[selectCardType["areaPos2"] = 2] = "areaPos2";
        selectCardType[selectCardType["areaPos3"] = 3] = "areaPos3";
        selectCardType[selectCardType["areaPos4"] = 4] = "areaPos4";
        selectCardType[selectCardType["nextCard"] = 5] = "nextCard";
        selectCardType[selectCardType["readyCard"] = 6] = "readyCard";
    })(selectCardType = core.selectCardType || (core.selectCardType = {}));
    class GameLogic {
        constructor() {
        }
        initGame(gameView) {
            this._gameView = gameView;
            this.resetGameData();
            !core.GameDef.selfTest && this.readLocalGameData();
            this.initCard();
            this.startGame();
        }
        getCanSaveCardNum() {
            return this._gameData.freeItemNum.CardSaved;
        }
        getCanPaidCardNum() {
            return this._gameData.paidItemNum.CardSaved;
        }
        setCanSaveCardNum(n) {
            this._gameData.freeItemNum.CardSaved += n;
        }
        setBomeNum(n) {
            this._gameData.freeItemNum.Bomb += n;
            return this._gameData.freeItemNum.Bomb;
        }
        getBomeNum() {
            return this._gameData.freeItemNum.Bomb;
        }
        setOmnNum(n) {
            this._gameData.freeItemNum.OmnipotentCard += n;
            return this._gameData.freeItemNum.OmnipotentCard;
        }
        getOmnNum() {
            return this._gameData.freeItemNum.OmnipotentCard;
        }
        getCurScore() {
            return this._gameData.curScore;
        }
        resetGameData() {
            let item = core.Player.ItemType;
            this._gameData = {
                bI: 0,
                mairArr: [[], [], [], []],
                readyCard: null,
                nextCardArr: [],
                curScore: 0,
                level: 1,
                renewTimes: 1,
                freeItemNum: {
                    OmnipotentCard: core.GameDef.isRank ? core.GameDef.itemLevelParseNum(core.Player.Manager.rankItemLevel_get(item.OmnipotentCard)) : 1,
                    Bomb: core.GameDef.isRank ? core.GameDef.itemLevelParseNum(core.Player.Manager.rankItemLevel_get(item.Bomb)) : 1,
                    CardSaved: core.GameDef.isRank ? core.GameDef.itemLevelParseNum(core.Player.Manager.rankItemLevel_get(item.CardSaved)) : 1,
                },
                paidItemNum: {
                    OmnipotentCard: core.GameDef.isRank ? 1 : 1,
                    Bomb: core.GameDef.isRank ? 1 : 1,
                    CardSaved: core.GameDef.isRank ? 1 : 1
                },
            };
            this._aniStatus = [true, true, true, true, true, true];
            this._selectedType = 0;
            this._selectedCard = null;
        }
        startGame() {
            // XnhdAPI.hideBanner();
            this._RandPoolTempData = [];
            if (this._notInGuiding) {
                let length = 0;
                for (let i = 0; i < 4; i++) {
                    length += this._gameData.mairArr[i].length;
                }
                if (this._gameData.curScore === 0 && length === 0) {
                    for (let i = 0; i < 4; i++) {
                        for (let j = 0; j < 3; j++) {
                            let p = this.getNormalCard();
                            let column = this._gameData.mairArr[i];
                            if (p.p1 === 1) {
                                p.p1 = 2;
                            }
                            if (column.length > 0 && p.p1 === column[column.length - 1].p1) {
                                if (p.p1 > 2) {
                                    p.p1 = p.p1 / 2;
                                }
                                else {
                                    p.p1 = 4;
                                }
                            }
                            column.push(p);
                        }
                    }
                    this._gameData.nextCardArr.push(this.getNormalCard());
                    this._gameData.nextCardArr.push(this.getNormalCard());
                    this.saveGameData();
                }
            }
            else {
                this._gameData.mairArr = [
                    [],
                    [{ id: 1, p1: 2, p2: 0, mk: 0 }, { id: 2, p1: 4, p2: 0, mk: 0 }, { id: 3, p1: 8, p2: 0, mk: 0 }],
                    [{ id: 4, p1: 128, p2: 0, mk: 0 }, { id: 5, p1: 64, p2: 0, mk: 0 }, { id: 12, p1: 32, p2: 0, mk: 0 }, { id: 6, p1: 16, p2: 0, mk: 0 }, { id: 7, p1: 4, p2: 0, mk: 0 }],
                    [{ id: 8, p1: 8, p2: 1, mk: 0 }]
                ];
                this._gameData.nextCardArr = [
                    { id: 9, p1: 4, p2: 0, mk: 0 },
                    { id: 10, p1: 8, p2: 0, mk: 0 }
                ];
                this._guideNextArr = [
                    { id: 11, p1: 1, p2: 0, mk: 0 }
                ];
                this._gameData.bI = 13;
            }
            this._gameView.resetOmnipotentCardNum(this._gameData.freeItemNum.OmnipotentCard, this._gameData.paidItemNum.OmnipotentCard);
            this._gameView.resetCardSaveTimes(this._gameData.freeItemNum.CardSaved, this._gameData.paidItemNum.CardSaved);
            this._gameView.resetBombNum(this._gameData.freeItemNum.Bomb, this._gameData.paidItemNum.Bomb);
            this._gameView.resetCoinNum();
            this._gameView.setLevel(this._gameData.level, this.getGoalCard());
            this.fillCard();
            this._gameView.resetGameArea(this._gameData, !this._notInGuiding);
            if (this.gameOverTest()) {
                this.gameOver();
            }
            else {
                this._gameView.allowPlayerPress(true);
                this.saveGameData();
            }
        }
        finishGuiding() {
            this.dealWithLevelUp(() => {
                this._notInGuiding = true;
                XnhdAPI.saveDataToCache('Guide', 'Guide');
            });
        }
        getNormalCard() {
            let a = this.getRandCard();
            while (a.p1 === 1) {
                a = this.getRandCard();
            }
            return a;
        }
        fillCard() {
            while (this._gameData.nextCardArr.length < 2) {
                this._gameData.nextCardArr.push(this.getNormalCard());
                this._aniStatus[4] = false;
                this._gameView.showNextCardAni(this._gameData.nextCardArr, () => {
                    this._aniStatus[4] = true;
                    this.saveGameData();
                });
            }
        }
        selectWhitchCard(id) {
            this._selectedCard = null;
            this._selectedType = -1;
            this.fillCard();
            if (id === this._gameData.nextCardArr[0].id) {
                if (this._aniStatus[4]) {
                    this._selectedType = selectCardType.nextCard;
                    this._selectedCard = this._gameData.nextCardArr[0];
                }
            }
            else if (this._gameData.readyCard && id === this._gameData.readyCard.id) {
                this._selectedType = selectCardType.readyCard;
                this._selectedCard = this._gameData.readyCard;
            }
            else {
                let I = -1;
                for (let i = 0; i < 4; i++) {
                    for (let j = 0; j < this._gameData.mairArr[i].length; j++) {
                        if (this._gameData.mairArr[i][j].id === id) {
                            I = i;
                            break;
                        }
                    }
                    if (I > -1) {
                        break;
                    }
                }
                if (I > -1) {
                    this._selectedType = I + 1;
                    this._selectedCard = this._gameData.mairArr[I][this._gameData.mairArr[I].length - 1];
                }
            }
            if (this._selectedType > 0 && this._aniStatus[this._selectedType - 1] && this._selectedCard) {
                this._gameView.pickOneCard(this._selectedCard.id);
            }
        }
        readyForUseReadyCard(successCallback, failCallback) {
            console.log("run in readyForUseReadyCard0");
            if (!this._notInGuiding) {
                console.log("run in readyForUseReadyCard1");
                successCallback();
            }
            else if (this._gameData.freeItemNum.CardSaved > 0 || core.GameDef.selfTest) {
                console.log("run in readyForUseReadyCard2");
                !core.GameDef.selfTest && this._gameData.freeItemNum.CardSaved--;
                this._gameView.resetCardSaveTimes(this._gameData.freeItemNum.CardSaved, this._gameData.paidItemNum.CardSaved);
                successCallback();
            }
            else if (this._gameData.paidItemNum.CardSaved > 0) {
                console.log("run in readyForUseReadyCard3");
                this._gameView.showNoSavedCard(() => {
                    console.log("run in readyForUseReadyCard3.1");
                    this._gameData.paidItemNum.CardSaved--;
                    this._gameView.resetCardSaveTimes(this._gameData.freeItemNum.CardSaved, this._gameData.paidItemNum.CardSaved);
                    successCallback();
                }, () => {
                    console.log("run in readyForUseReadyCard3.2");
                    failCallback();
                });
            }
            else {
                console.log("run in readyForUseReadyCard4, this._gameData.paidItemNum.CardSaved:", this._gameData.paidItemNum.CardSaved);
                // this.cancelSelectCard();
                this._gameView.showNoSavedCard(() => {
                    console.log("run in readyForUseReadyCard4.1");
                    // this._gameData.paidItemNum.CardSaved--;
                    this._gameView.resetCardSaveTimes(this._gameData.freeItemNum.CardSaved, this._gameData.paidItemNum.CardSaved);
                    successCallback();
                }, () => {
                    console.log("run in readyForUseReadyCard4.2");
                    failCallback();
                });
            }
            console.log("run in readyForUseReadyCard9");
        }
        useOmnipotentCard() {
            if (!this._notInGuiding) {
                this._useOmnipotentCard();
            }
            else if (this._aniStatus[4]) {
                if (this.GetPutCardInfoCount() > 0) {
                    if (this._gameData.freeItemNum.OmnipotentCard > 0 || core.GameDef.selfTest) {
                        !core.GameDef.selfTest && this._gameData.freeItemNum.OmnipotentCard--;
                        this._useOmnipotentCard();
                    }
                    else if (this._gameData.freeItemNum.OmnipotentCard === 0) {
                        this._gameView.resetOmnipotentCardNum(this._gameData.freeItemNum.OmnipotentCard, this._gameData.paidItemNum.OmnipotentCard);
                        // XnhdAPI.showRewardAd(2);
                        XnhdAPI.showVideoAd("wannengpai", () => {
                            console.log("jlog, useOmnipotentCard.showVideoAd.succ0");
                            // core.GameLogic.Default.setOmnNum(1);//加一个万能牌
                            core.GameLogic.Default._useOmnipotentCard();
                            console.log("jlog, useOmnipotentCard.showVideoAd.succ1");
                        }, f => {
                            console.log("jlog, useOmnipotentCard.showVideoAd.fail0");
                            XnhdAPI.showTips(core.GameDef.localizationConfig.finishVideoTips[core.GameDef.languageIndex]);
                            console.log("jlog, useOmnipotentCard.showVideoAd.fail1");
                        }, f => {
                            console.log("jlog, useOmnipotentCard.showVideoAd.noad0");
                            XnhdAPI.showTips(core.GameDef.localizationConfig.noVideoTips[core.GameDef.languageIndex]);
                            console.log("jlog, useOmnipotentCard.showVideoAd.noad1");
                        });
                        //this._gameData.freeItemNum.OmnipotentCard += 1;
                        //this._useOmnipotentCard()
                    }
                    else if (this._gameData.paidItemNum.OmnipotentCard > 0) {
                        this._gameView.showNoOmnipotentCard(() => {
                            this._gameData.paidItemNum.OmnipotentCard--;
                            this._useOmnipotentCard();
                        }, null);
                    }
                }
            }
        }
        _useOmnipotentCard() {
            this.fillCard();
            this._gameData.nextCardArr[0].p1 = 1;
            this._gameData.nextCardArr[1].p1 = 1;
            this._aniStatus[4] = false;
            this._gameView.nextCardChange(this._gameData.nextCardArr, () => {
                this._aniStatus[4] = true;
                this._gameView.resetOmnipotentCardNum(this._gameData.freeItemNum.OmnipotentCard, this._gameData.paidItemNum.OmnipotentCard);
            });
            this.saveGameData();
        }
        useBomb() {
            let fIndex = this._aniStatus.indexOf(false);
            if (fIndex > 3 || fIndex === -1) {
                this.fillCard();
                if (this._gameData.nextCardArr[0].p1 != 1) {
                    if (this._gameData.freeItemNum.Bomb > 0 || core.GameDef.selfTest) {
                        !core.GameDef.selfTest && this._gameData.freeItemNum.Bomb--;
                        this._useBomb();
                    }
                    else if (this._gameData.freeItemNum.Bomb === 0) {
                        this._gameView.resetBombNum(this._gameData.freeItemNum.Bomb, this._gameData.paidItemNum.Bomb);
                        // XnhdAPI.showRewardAd(3);
                        XnhdAPI.showVideoAd("zhadan", () => {
                            console.log("jlog, useOmnipotentCard.useBomb.succ0");
                            // core.GameLogic.Default.setBomeNum(1);//如果播放结束，加一个炸弹
                            core.GameLogic.Default._useBomb();
                            console.log("jlog, useOmnipotentCard.useBomb.succ1");
                        }, f => {
                            console.log("jlog, useOmnipotentCard.useBomb.fail0");
                            XnhdAPI.showTips(core.GameDef.localizationConfig.finishVideoTips[core.GameDef.languageIndex]);
                            console.log("jlog, useOmnipotentCard.useBomb.fail1");
                        }, f => {
                            console.log("jlog, useOmnipotentCard.useBomb.noad0");
                            XnhdAPI.showTips(core.GameDef.localizationConfig.noVideoTips[core.GameDef.languageIndex]);
                            console.log("jlog, useOmnipotentCard.useBomb.noad1");
                        });
                        //这些操作都放置到广告成功回调里面处理。（因为广告有延迟响应）
                        // this._gameData.freeItemNum.Bomb+=1;
                        //this._useBomb();
                    }
                    else if (this._gameData.paidItemNum.Bomb > 0) {
                        this._gameView.showNoBomb(() => {
                            this._gameData.paidItemNum.Bomb--;
                            this._useBomb();
                        }, null);
                    }
                }
                else {
                    let l = core.GameDef.localizationConfig;
                    let i = core.GameDef.languageIndex;
                    let oI = l.GameView.cannotUseBomb[i].indexOf('{OmnipotentCard}');
                    let oL = '{OmnipotentCard}'.length;
                    let bI = l.GameView.cannotUseBomb[i].indexOf('{Bomb}');
                    let bL = '{Bomb}'.length;
                    let a;
                    if (oI < bI) {
                        a = l.GameView.cannotUseBomb[i].substring(0, oI) + l.itemDescription.OmnipotentCard[i];
                        a += l.GameView.cannotUseBomb[i].substring(oI + oL, bI) + l.itemDescription.Bomb[i];
                        a += l.GameView.cannotUseBomb[i].substring(bL + bI);
                    }
                    else {
                        a = l.GameView.cannotUseBomb[i].substring(0, bI) + l.itemDescription.Bomb[i];
                        a += l.GameView.cannotUseBomb[i].substring(bI + bL, oI) + l.itemDescription.OmnipotentCard[i];
                        a += l.GameView.cannotUseBomb[i].substring(oI + oL);
                    }
                    XnhdAPI.showTips(a);
                }
            }
        }
        _useBomb() {
            let a = this._gameData.mairArr;
            this._gameData.mairArr = [[], [], [], []];
            let scoreArr = [];
            for (let i = 0; i < a.length; i++) {
                for (let j = 0; j < a[i].length; j++) {
                    this._gameData.curScore += a[i][j].p1 * this._gameData.level;
                    scoreArr.push(this._gameData.curScore);
                }
            }
            this._gameView.useBombEff(a, scoreArr);
            this._gameView.resetBombNum(this._gameData.freeItemNum.Bomb, this._gameData.paidItemNum.Bomb);
            this.saveGameData();
        }
        putDownCard_test(x, y, guideStep) {
            let areaParam = core.GameDef.gameParam.gameArea;
            let i = Math.floor((x - areaParam.x0 + core.GameDef.gameParam.cardSize.s.width / 2) / areaParam.x_Space);
            let showFlashArr = [];
            if (guideStep === 0) {
                if (y < areaParam.y_max && i === 2) {
                    showFlashArr = ['9', '7'];
                }
            }
            else if (guideStep === 1) {
                if (y < areaParam.y_max && i === 2) {
                    showFlashArr = ['10', '7'];
                }
            }
            else if (guideStep === 2) {
                if (y < areaParam.y_max && i === 2) {
                    showFlashArr = ['11', '4']; //测试可放扑克
                }
            }
            else if (guideStep === 3) {
                if (y < areaParam.y_max && i === 1) {
                    showFlashArr = ['3', '8'];
                }
            }
            else if (y < areaParam.y_max && i > -1 && i < 4 && this._aniStatus[i] && this._selectedType > 0) {
                let canShowFlash = false;
                if (this._selectedType < 5) {
                    if (this._selectedType != i + 1 && this._gameData.mairArr[i].length > 0 && (this._gameData.mairArr[i][this._gameData.mairArr[i].length - 1].p1 === this._selectedCard.p1)) {
                        canShowFlash = true;
                    }
                }
                else if (this._selectedCard.p1 === 1) {
                    if (this._gameData.mairArr[i].length > 0 && this._gameData.mairArr[i][this._gameData.mairArr[i].length - 1].p1 != 1) {
                        canShowFlash = true;
                    }
                }
                else if (this._gameData.mairArr[i].length != 8 || this._gameData.mairArr[i][this._gameData.mairArr[i].length - 1].p1 === this._selectedCard.p1) {
                    canShowFlash = true;
                }
                if (canShowFlash) {
                    if (this._gameData.mairArr[i].length > 0) {
                        showFlashArr = [this._selectedCard.id.toString(), this._gameData.mairArr[i][this._gameData.mairArr[i].length - 1].id.toString()];
                    }
                    else {
                        showFlashArr = [this._selectedCard.id.toString(), 'back_' + (i + 1)];
                    }
                }
            }
            else if (this._selectedType === 5 && Math.abs(x - 596 - core.GameDef.gameParam.gameArea.x_Adjust) < core.GameDef.gameParam.cardSize.s.width / 2 && Math.abs(y - 867 + core.GameDef.gameParam.gameArea.y_Adjust) < core.GameDef.gameParam.cardSize.s.height / 2) {
                showFlashArr = [this._selectedCard.id.toString()];
            }
            this._gameView.showCardFlash(showFlashArr);
        }
        randPutNextPoker() {
            let areaParam = core.GameDef.gameParam.gameArea;
            let canPutI = [];
            this.fillCard();
            let nextCard = this._gameData.nextCardArr[0];
            for (let i = 0; i < 4; i++) {
                let column = this._gameData.mairArr[i];
                if (this._aniStatus[i] && (nextCard.p1 === 1 || column.length < 8 || column[column.length - 1].p1 === nextCard.p1)) {
                    canPutI.push(i);
                }
            }
            if (canPutI.length > 0) {
                canPutI.sort((a, b) => {
                    return Math.random() * 2 - 1;
                });
                this.cancelSelectCard();
                this._selectedType = selectCardType.nextCard;
                let x = canPutI[0] * areaParam.x_Space + areaParam.x0 + core.GameDef.gameParam.cardSize.s.width / 2;
                this._selectedCard = nextCard;
                this._gameView.pickOneCard(this._gameData.nextCardArr[0].id);
                this.putDownCard(x, 50, -1);
            }
            else {
                this.gameOver();
            }
        }
        putDownCard(x, y, guideStep) {
            let areaParam = core.GameDef.gameParam.gameArea;
            let i = Math.floor((x - areaParam.x0 + core.GameDef.gameParam.cardSize.s.width / 2) / areaParam.x_Space);
            if (y < areaParam.y_max && i > -1 && i < 4 && this._aniStatus[i]) {
                let theCard;
                if (this._selectedType === selectCardType.nextCard && (this._gameData.mairArr[i].length != 8 || this._gameData.mairArr[i][this._gameData.mairArr[i].length - 1].p1 === this._selectedCard.p1 || this._selectedCard.p1 === 1)) {
                    if ((guideStep === 0 && i === 2) || (guideStep === 1 && i === 2) || (guideStep === 2 && i === 2)) {
                        //从手牌放扑克
                        theCard = this._gameData.nextCardArr.shift();
                    }
                    else if (guideStep < 0) {
                        let m = false;
                        if (this._selectedCard.p1 === 1) {
                            if (this._gameData.mairArr[i].length > 0 && this._gameData.mairArr[i][this._gameData.mairArr[i].length - 1].p1 != 1) {
                                m = true;
                            }
                        }
                        else if (this._gameData.mairArr[i].length != 8) {
                            m = true;
                        }
                        else if (this._gameData.mairArr[i][this._gameData.mairArr[i].length - 1].p1 === this._selectedCard.p1) {
                            m = true;
                        }
                        if (m) {
                            theCard = this._gameData.nextCardArr.shift();
                        }
                    }
                }
                else if (this._selectedType === selectCardType.readyCard) {
                    let m = false;
                    if (this._selectedCard.p1 === 1) {
                        if (this._gameData.mairArr[i].length > 0 && this._gameData.mairArr[i][this._gameData.mairArr[i].length - 1].p1 != 1) {
                            m = true;
                        }
                    }
                    else if (this._gameData.mairArr[i].length != 8) {
                        m = true;
                    }
                    else if (this._gameData.mairArr[i][this._gameData.mairArr[i].length - 1].p1 === this._selectedCard.p1) {
                        m = true;
                    }
                    if (m) {
                        theCard = this._gameData.readyCard;
                        this._gameData.readyCard = null;
                    }
                }
                else if (this._selectedType > 0) {
                    let canPutDown = false;
                    if (this._selectedType < 5) {
                        if (guideStep < 0 && this._selectedType != i + 1 && this._gameData.mairArr[i].length > 0 && this._gameData.mairArr[i][this._gameData.mairArr[i].length - 1].p1 === this._selectedCard.p1) {
                            canPutDown = true;
                        }
                        else if (guideStep === 3 && i === 1) {
                            //从桌面放扑克
                            canPutDown = true;
                        }
                    }
                    if (canPutDown) {
                        theCard = this._gameData.mairArr[this._selectedType - 1].pop();
                    }
                }
                if (theCard) {
                    this._gameData.mairArr[i].push(theCard);
                    this._gameView.putCardDown(i, this._gameData.mairArr[i].length - 1, 0);
                    this.eliminateForOneColumn(i, () => {
                        if (this.gameOverTest()) {
                            this.gameOver();
                        }
                        else {
                            this._gameView.allowPlayerPress(true);
                            this.saveGameData();
                        }
                    });
                }
                else {
                    this.cancelSelectCard();
                }
            }
            else if (Math.abs(x - 596 - core.GameDef.gameParam.gameArea.x_Adjust) < core.GameDef.gameParam.cardSize.s.width / 2 && Math.abs(y - 867 + core.GameDef.gameParam.gameArea.y_Adjust) < core.GameDef.gameParam.cardSize.s.height / 2 && !this._gameData.readyCard && this._selectedType === selectCardType.nextCard) {
                if (guideStep < 0) {
                    console.log("readyCard 1");
                    this.readyForUseReadyCard(() => {
                        console.log("readyCard 2");
                        this._gameData.readyCard = this._gameData.nextCardArr.shift();
                        this._gameView.putCardReady(0);
                        this._gameData.nextCardArr.push(this.getRandCard());
                        this._aniStatus[4] = false;
                        this._gameView.showNextCardAni(this._gameData.nextCardArr, () => {
                            if (this.gameOverTest()) {
                                this.gameOver();
                            }
                            else {
                                this._gameView.allowPlayerPress(true);
                                this._aniStatus[4] = true;
                                this.saveGameData();
                            }
                        });
                    }, () => {
                        console.log("readyCard 3");
                        this.cancelSelectCard();
                    });
                }
                else {
                    console.log("readyCard 4");
                    this.cancelSelectCard();
                }
                return;
            }
            else {
                this.cancelSelectCard();
            }
            if (this._gameData.nextCardArr.length === 1) {
                this._gameData.nextCardArr.push(this.getRandCard());
                // this._aniStatus[4] = false;
                //         this._gameView.showNextCardAni(this._gameData.nextCardArr, () => {
                //             if (this.gameOverTest()) {
                //                 this.gameOver();
                //             } else {
                //                 this._gameView.allowPlayerPress(true);
                //                 this._aniStatus[4] = true;
                //                 this.saveGameData();
                //             }
                //         })
                this._aniStatus[4] = false;
                this._gameView.showNextCardAni(this._gameData.nextCardArr, () => {
                    this._aniStatus[4] = true;
                    this.saveGameData();
                });
            }
        }
        eliminateForOneColumn(column, eliCallback) {
            let a = this.eliminateColumn(column);
            if (a.length > 0) {
                this._aniStatus[column] = false;
                let lastCard = this._gameData.mairArr[column][this._gameData.mairArr[column].length - 1];
                // if (lastCard.p1 === this.getGoalCard()) {
                //     this._gameData.level++;
                // }
                let scoreArr = [];
                let m = 0;
                for (; m < a.length; m++) {
                    this._gameData.curScore += this.caculateScore(a[m], m + 1);
                    scoreArr.push(this._gameData.curScore);
                }
                let levelUpCard = this.getLevelUpCard(column);
                if (levelUpCard) {
                    this._gameData.curScore += this.caculateScore(levelUpCard, m + 1);
                    var levelUpScore = this._gameData.curScore;
                    this._gameData.level++;
                    if (core.GameDef.isRank) {
                        let bgSkinArr = core.GameDef.bgParam;
                        for (let i = 0; i < bgSkinArr.length; i++) {
                            if (this._gameData.level === bgSkinArr[i].param1) {
                                core.Player.Manager.unlockSkin_bg(i);
                                break;
                            }
                        }
                    }
                }
                this._gameView.eliminateCards(a, scoreArr, lastCard, () => {
                    this._aniStatus[column] = true;
                    if (levelUpCard) {
                        this._gameView.showLevelUp(levelUpCard, levelUpScore, this._gameData.level - 1, res => {
                            if (this._notInGuiding) {
                                if (res.name === core.Player.ItemType.Coin) {
                                    console.log('dsAd-jlog, run in adjustCoin3Num');
                                    core.Player.Manager.adjustCoinNum(res.num);
                                    this._gameView.resetCoinNum();
                                }
                                else if (res.name === core.Player.ItemType.CardSaved) {
                                    this._gameData.freeItemNum.CardSaved += res.num;
                                    this._gameView.resetCardSaveTimes(this._gameData.freeItemNum.CardSaved, this._gameData.paidItemNum.CardSaved);
                                }
                                else if (res.name === core.Player.ItemType.Bomb) {
                                    this._gameData.freeItemNum.Bomb += res.num;
                                    this._gameView.resetBombNum(this._gameData.freeItemNum.Bomb, this._gameData.paidItemNum.Bomb);
                                }
                                else if (res.name === core.Player.ItemType.OmnipotentCard) {
                                    this._gameData.freeItemNum.OmnipotentCard += res.num;
                                    this._gameView.resetOmnipotentCardNum(this._gameData.freeItemNum.OmnipotentCard, this._gameData.paidItemNum.OmnipotentCard);
                                }
                                this._gameView.setLevel(this._gameData.level, this.getGoalCard());
                                this.dealWithLevelUp(() => {
                                    eliCallback && eliCallback(true);
                                });
                            }
                            else {
                                eliCallback && eliCallback(true);
                            }
                        });
                    }
                    else {
                        eliCallback && eliCallback(true);
                    }
                });
            }
            else {
                eliCallback && eliCallback(false);
            }
        }
        dealWithLevelUp(callBack) {
            let a = this.levelUpAddCards();
            if (a.length > 0) {
                this._gameView.levelUpAddCard(a, () => {
                    let m = 4;
                    for (let n = 0; n < 4; n++) {
                        this.eliminateForOneColumn(n, () => {
                            m--;
                            if (m === 0) {
                                callBack && callBack();
                            }
                        });
                    }
                });
            }
            else {
                callBack && callBack();
            }
        }
        levelUpAddCards() {
            let addCardNum = this.getLevelUpCardNum();
            if (!this._notInGuiding) {
                addCardNum = 12;
            }
            let addCardArr = [];
            while (addCardArr) {
                for (let i = 0; i < this._gameData.mairArr.length; i++) {
                    if (addCardNum > 0 && this._gameData.mairArr[i].length < 8) {
                        let card = this.getNormalCard();
                        let lastCard = this._gameData.mairArr[i][this._gameData.mairArr[i].length - 1];
                        if (card.p1 === 1) {
                            card.p1 = 2;
                        }
                        if (lastCard && card.p1 === lastCard.p1) {
                            if (lastCard.p1 === 2) {
                                card.p1 = 4;
                            }
                            else {
                                card.p1 = lastCard.p1 / 2;
                            }
                        }
                        addCardArr.push({ ij: [i, this._gameData.mairArr[i].length], card: card });
                        this._gameData.mairArr[i].push(card);
                        addCardNum--;
                    }
                    else if (addCardNum === 0 || this.GetPutCardInfoCount() === 32) {
                        return addCardArr;
                    }
                }
            }
            return addCardArr;
        }
        getLevelUpCard(i) {
            if (this._gameData.mairArr[i].length > 0 && this._gameData.mairArr[i][this._gameData.mairArr[i].length - 1].p1 === this.getGoalCard()) {
                return this._gameData.mairArr[i].pop();
            }
            return null;
        }
        gameOverTest() {
            let gameOver = true;
            this.fillCard();
            if (this.GetPutCardInfoCount() < 32) {
                gameOver = false;
            }
            else if (!this._gameData.readyCard && this._gameData.freeItemNum.CardSaved > 0) {
                gameOver = false;
            }
            else if (this._gameData.nextCardArr[0].p1 === 1) {
                gameOver = false;
            }
            else {
                let p1Arr = [];
                for (let i = 0; i < 4; i++) {
                    let p1 = this._gameData.mairArr[i][7].p1;
                    if (p1 === this._gameData.nextCardArr[0].p1 || (this._gameData.readyCard && p1 === this._gameData.readyCard.p1)) {
                        gameOver = false;
                        break;
                    }
                    if (p1Arr.indexOf(p1) > -1) {
                        gameOver = false;
                        break;
                    }
                    else {
                        p1Arr.push(p1);
                    }
                }
            }
            return gameOver;
        }
        gameOver() {
            this._gameView.allowPlayerPress(false);
            if (this._gameData.freeItemNum.Bomb > 0) {
                this._gameView.showRenewLayer(true, e => {
                    if (e) {
                        this._gameData.freeItemNum.Bomb--;
                        this._useBomb();
                    }
                    else {
                        this._gameView.showGameOver();
                    }
                });
            }
            else if (this._gameData.paidItemNum.Bomb > 0) {
                this._gameView.showRenewLayer(false, e => {
                    if (e) {
                        this._gameData.paidItemNum.Bomb--;
                        this._useBomb();
                    }
                    else {
                        this._gameView.showGameOver();
                    }
                });
            }
            else {
                this._gameView.showGameOver();
            }
        }
        cancelSelectCard() {
            let n = -1;
            if (this._selectedType > 0 && this._selectedType != selectCardType.readyCard && this._selectedType != selectCardType.nextCard) {
                for (let j = 0; j < this._gameData.mairArr[this._selectedType - 1].length; j++) {
                    if (this._gameData.mairArr[this._selectedType - 1][j].id === this._selectedCard.id) {
                        n = j;
                        break;
                    }
                }
            }
            this._gameView.cancelSelect(this._selectedType, n);
            this._selectedType = -1;
            this._selectedCard = null;
        }
        eliminateColumn(whitchColumn) {
            if (whitchColumn > -1 && whitchColumn < 4) {
                let thisColumn = this._gameData.mairArr[whitchColumn];
                let eliminateArr = [];
                let hasBomb = false;
                while (thisColumn.length > 1) {
                    let thisCard = thisColumn[thisColumn.length - 1];
                    let upperCard = thisColumn[thisColumn.length - 2];
                    if (thisCard.p1 === upperCard.p1 || thisCard.p1 === 1) {
                        if (upperCard.p2 === 1 || thisCard.p2 === 1) {
                            hasBomb = true;
                        }
                        else {
                            upperCard.p1 *= 2;
                        }
                        eliminateArr.push(thisColumn.pop());
                    }
                    else {
                        break;
                    }
                }
                if (hasBomb) {
                    while (thisColumn.length > 0) {
                        eliminateArr.push(thisColumn.pop());
                    }
                }
                return eliminateArr;
            }
            return [];
        }
        saveGameData() {
            this._notInGuiding && XnhdAPI.saveDataToCache(core.GameDef.isRank ? "RankData" : "GameData", JSON.stringify(this._gameData));
        }
        readLocalGameData() {
            let data = XnhdAPI.readDataFromCache(core.GameDef.isRank ? "RankData" : "GameData");
            if (data && data.length > 0) {
                try {
                    let a = JSON.parse(data);
                    this._gameData.bI = a.bI;
                    this._gameData.curScore = a.curScore;
                    this._gameData.mairArr = a.mairArr;
                    this._gameData.readyCard = a.readyCard;
                    this._gameData.nextCardArr = a.nextCardArr;
                    this._gameData.level = a.level;
                    this._gameData.renewTimes = a.renewTimes;
                    this._gameData.freeItemNum.OmnipotentCard = a.freeItemNum.OmnipotentCard;
                    this._gameData.freeItemNum.Bomb = a.freeItemNum.Bomb;
                    this._gameData.freeItemNum.CardSaved = a.freeItemNum.CardSaved;
                    this._gameData.paidItemNum.Bomb = a.paidItemNum.Bomb;
                    this._gameData.paidItemNum.OmnipotentCard = a.paidItemNum.OmnipotentCard;
                    this._gameData.paidItemNum.CardSaved = a.paidItemNum.CardSaved;
                    this._notInGuiding = true;
                }
                catch (e) { }
            }
            else {
                let needGuide = XnhdAPI.readDataFromCache('Guide');
                if (needGuide) {
                    this._notInGuiding = true;
                }
                else {
                    if (!core.GameDef.isRank) {
                        this._notInGuiding = false;
                    }
                    else {
                        this._notInGuiding = true;
                    }
                    XnhdAPI.logDebug('新手玩家');
                }
            }
        }
        getGoalCard() {
            let a = this._gameData.level;
            let b = core.GameDef.gameParam.level;
            if (a > b.length) {
                return 8192;
            }
            return b[a - 1];
        }
        getLevelUpCardNum() {
            let a = this._gameData.level;
            let b = core.GameDef.gameParam.flyCardNum;
            if (a > b.length) {
                return b[b.length - 1];
            }
            return b[a - 1];
        }
        initCard() {
            this._cardRand = [];
            let b = core.GameDef.gameParam.card;
            for (let i = 0; i < b.RandType.length; i++) {
                let a = {
                    RandType: b.RandType[i],
                    RandNumber: b.RandNumber[i],
                    RandCdId: b.RandCdId[i],
                    RandCd: b.RandCd[i],
                    ConditionMinDouble: b.ConditionMinDouble[i],
                    ConditionMaxDouble: b.ConditionMaxDouble[i],
                    RatePut0_8: b.RatePut0_8[i],
                    RatePut9_16: b.RatePut9_16[i],
                    RatePut17_24: b.RatePut17_24[i],
                    RatePut25_32: b.RatePut25_32[i],
                    AddRateFactor: b.AddRateFactor[i],
                    RateLower: b.RateLower[i],
                    RateUpper: b.RateUpper[i],
                };
                this._cardRand.push(a);
            }
        }
        canGetShit(I) {
            // this._gameData.nextCardArr 手牌
            // this._gameData.mairArr 牌桌
            // this.getGoalCard() 当前最大牌
            var a = [2, 4, 8, 16, 32, 64, 128, 256, 512, 1024, 2048];
            let gailv = 0.2 + 0.02 * this._gameData.level;
            var r = Math.random();
            if (r >= gailv) {
                console.log("nongshaliecard?, r:", r, ", gailv:", gailv);
                return I;
            }
            console.log("cur level:", this._gameData.level);
            while (a.length > 0) {
                let pos = Math.floor(Math.random() * a.length);
                let v = a[pos];
                a.splice(pos, 1);
                if (v >= this.getGoalCard())
                    continue;
                var flag = false;
                for (var i = 0; i < this._gameData.nextCardArr.length; ++i) {
                    if (v == this._gameData.nextCardArr[i].p1) {
                        flag = true;
                        break;
                    }
                }
                for (var i = 0; i < this._gameData.mairArr.length; ++i) {
                    if (this._gameData.mairArr[i].length > 0 && v == this._gameData.mairArr[i][this._gameData.mairArr[i].length - 1].p1) {
                        flag = true;
                        break;
                    }
                }
                if (!flag) {
                    console.log("vvv card:", v, ", src:", I);
                    I.p1 = v;
                    return I;
                }
            }
            console.log("vvvz card, ret:", I);
            return I;
        }
        getRandCard() {
            if (this._guideNextArr && this._guideNextArr.length > 0) {
                return this._guideNextArr.shift();
            }
            else {
                for (var e = this._gameData.bI, t = this._cardRand, n = 0, a = [], i = this.GetPutCardInfoCount(), o = this._gameData.level - 1, r = 0, c = t.length; r < c; ++r) {
                    var s = t[r], l = this._RandPoolTempData[s.RandCdId];
                    if (!(null != l && l > e)) {
                        var d = o + 1, u = s.ConditionMinDouble;
                        if (!(-1 != u && d < u)) {
                            var h = s.ConditionMaxDouble;
                            if (!(-1 != h && d > h)) {
                                var f;
                                f = i >= 0 && i <= 8 ? s.RatePut0_8 : i >= 9 && i <= 16 ? s.RatePut9_16 : i >= 17 && i <= 24 ? s.RatePut17_24 : s.RatePut25_32;
                                var g = s.AddRateFactor * o, C = f + g;
                                if (0 != C) {
                                    0 != g && (C < s.RateLower ? C = s.RateLower : C > s.RateUpper && (C = s.RateUpper));
                                    var p = n + C;
                                    let as = {
                                        MinRateRange: n,
                                        MaxRateRange: p,
                                        RandInfo: s
                                    };
                                    a.push(as), n = p;
                                }
                            }
                        }
                    }
                }
                var S = Math.random() * n, v = null;
                for (r = 0, c = a.length; r < c; ++r) {
                    let s = a[r];
                    if (S >= s.MinRateRange && S <= s.MaxRateRange) {
                        v = s.RandInfo;
                        break;
                    }
                }
                null == v && (v = t[0], XnhdAPI.logError("随机失败补充: 当前张数" + i + ", 当前范围" + S + ", 最大范围:" + n + ", 总随机次:" + e + ", RandNumber:" + v.RandNumber + ", RandType:" + v.RandType));
                var I = this.NewCardInfo(v.RandNumber, v.RandType);
                this._RandPoolTempData[v.RandCdId] = e + v.RandCd;
                return this.canGetShit(I);
            }
        }
        NewCardInfo(_num, _type) {
            if (_type > 1) {
                XnhdAPI.logDebug('特殊牌：' + _num + '，type=' + _type);
            }
            let a = {
                id: ++this._gameData.bI,
                p1: _type === 3 ? 1 : _num,
                p2: _type === 2 ? 1 : 0,
                mk: 0
            };
            return a;
        }
        GetPutCardInfoCount() {
            let a = 0;
            for (let i = 0; i < this._gameData.mairArr.length; i++) {
                a += this._gameData.mairArr[i].length;
            }
            return a;
        }
        caculateScore(prop, times) {
            return (prop.p1 > 1 ? prop.p1 : 1) * times * this._gameData.level;
        }
    }
    GameLogic.Default = new GameLogic;
    core.GameLogic = GameLogic;
})(core || (core = {}));
