/**
 * 控制声音
 */
var core;
(function (core) {
    var AudioAndSetting = /** @class */ (function () {
        function AudioAndSetting() {
            this._soundPathArr = [
                "unpack/sound/click",
                "unpack/sound/put",
                "unpack/sound/putdown",
                "unpack/sound/levelUp",
                "unpack/sound/gameOver",
                "unpack/sound/bomb",
                "unpack/sound/hugeReward1",
                "unpack/sound/hugeReward2",
            ];
            this._music = "unpack/sound/bg";
            this.volumeMusic = 70;
            this.volumeSound = 100;
            this.vibrate = true;
        }
        AudioAndSetting.prototype.sound_putDown = function () {
            this.playSound(this._soundPathArr[2]);
        };
        AudioAndSetting.prototype.soundClick = function () {
            this.playSound(this._soundPathArr[0]);
        };
        AudioAndSetting.prototype.sound_pickUp = function () {
            this.playSound(this._soundPathArr[1]);
        };
        AudioAndSetting.prototype.sound_levelUp = function () {
            this.playSound(this._soundPathArr[3]);
        };
        AudioAndSetting.prototype.sound_gameOver = function () {
            this.playSound(this._soundPathArr[4]);
        };
        AudioAndSetting.prototype.sound_bomb = function () {
            this.playSound(this._soundPathArr[5]);
        };
        AudioAndSetting.prototype.sound_startTurntable = function () {
            this.playSound(this._soundPathArr[6]);
        };
        AudioAndSetting.prototype.sound_stopTurntable = function () {
            this.playSound(this._soundPathArr[7]);
        };
        AudioAndSetting.prototype.soundCombo = function (num) {
            num > 8 && (num = 8);
            this.playSound("unpack/sound/eliminate_" + num.toString());
        };
        AudioAndSetting.prototype.soundCongratulate = function (num) {
            num > 5 && (num = 5);
            this.playSound("unpack/sound/congratulate_" + num.toString());
        };
        AudioAndSetting.prototype.playMusic = function () {
            XnhdAPI.Sound.Manager.playMusic(this._music);
        };
        AudioAndSetting.prototype.playSound = function (whitchSound) {
            XnhdAPI.Sound.Manager.playSound(whitchSound);
        };
        AudioAndSetting.prototype.init = function () {
            Laya.SoundManager.autoReleaseSound = false;
            var res = XnhdAPI.readDataFromCache("setting");
            if (res && res.length) {
                try {
                    var jsonData = JSON.parse(res);
                    if (jsonData) {
                        if (jsonData.music > 1 || jsonData.music === 0) {
                            this.volumeMusic = jsonData.music;
                        }
                        if (jsonData.sound > 1 || jsonData.sound === 0) {
                            this.volumeSound = jsonData.sound;
                        }
                        if (typeof jsonData.vibrate === 'boolean') {
                            this.vibrate = jsonData.vibrate;
                        }
                        if (typeof jsonData.lang === 'number' && jsonData.lang > -1) {
                            core.GameDef.languageIndex = jsonData.lang;
                        }
                    }
                }
                catch (e) {
                    XnhdAPI.logError("error: position: readLocalConfig >>" + e.toString());
                }
            }
            XnhdAPI.Sound.Manager.init(this._soundPathArr, this._music, this.volumeSound, this.volumeMusic);
        };
        AudioAndSetting.prototype.writeSettingConfig = function () {
            XnhdAPI.saveDataToCache("setting", JSON.stringify({
                music: this.volumeMusic,
                sound: this.volumeSound,
                vibrate: this.vibrate,
                lang: core.GameDef.languageIndex,
            }), false, function () {
                XnhdAPI.logDebug("save Audio");
            });
        };
        AudioAndSetting.prototype.setSoundVolume = function (value) {
            this.volumeSound = value;
            XnhdAPI.Sound.Manager.setSoundVolume(value);
            this.writeSettingConfig();
        };
        AudioAndSetting.prototype.setMusicVolume = function (value) {
            this.volumeMusic = value;
            XnhdAPI.Sound.Manager.setMusicVolume(value);
            this.writeSettingConfig();
        };
        AudioAndSetting.prototype.setVibrate = function (isVibrate) {
            this.vibrate = isVibrate;
            this.writeSettingConfig();
        };
        AudioAndSetting.prototype.stopMusic = function () {
            XnhdAPI.Sound.Manager.stopMusic();
        };
        AudioAndSetting.prototype.stopSound = function () {
            XnhdAPI.Sound.Manager.stopAllSound();
        };
        AudioAndSetting.prototype.stopAll = function () {
            XnhdAPI.Sound.Manager.stopAll();
        };
        AudioAndSetting.Manager = new AudioAndSetting();
        return AudioAndSetting;
    }());
    core.AudioAndSetting = AudioAndSetting;
})(core || (core = {}));
//# sourceMappingURL=AudioAndSetting.js.map